var EXPORTED_SYMBOLS = ["determineCharset"];

const Cc = Components.classes;
const Ci = Components.interfaces;

const loader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
loader.loadSubScript("chrome://global/content/nsUserSettings.js");

var filterList = [];

function addFilter(url, charset) {
	var uri = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newURI(url, charset, null);
	if (!filterList[uri.host]) filterList[uri.host] = [];
	filterList[uri.host][uri.spec] = uri.originCharset;
}

function loadFilterList() {
	delete filterList; filterList = [];
	var list = nsPreferences.copyUnicharPref("extensions.charsetswitcher.filterlist", "");
	var re = /(\S+),(\S+)/g;
	var m;
	while (m = re.exec(list)) {
		addFilter(m[1], m[2]);
	}
}

function saveFilterList() {
	var list = [];
	for each (let filter in filterList) {
		for (let uri in Iterator(filter)) {
			list.push(uri.join(","));
		}
	}
	list.sort();
	nsPreferences.setUnicharPref("extensions.charsetswitcher.filterlist", list.join(" "));
}

function determineCharset(docShell, newLocation) {
	if (!docShell.currentURI.equals(newLocation)) {
		docShell.forcedCharset = null;
	}
	var filter = filterList[newLocation.host];
	if (filter) {
		for (let [url, charset] in Iterator(filter)) {
			if (newLocation.spec.substring(0, url.length) == url) {
				if (docShell.forcedCharset) {
					addFilter(url, docShell.forcedCharset.toString());
					saveFilterList();
				} else {
					docShell.prepareForNewContentModel();
					docShell.QueryInterface(Ci.nsIDocCharset).charset = charset;
				}
				return;
			}
		}
	}
	if (docShell.forcedCharset) {
		var url = newLocation.spec.substring(0, newLocation.spec.lastIndexOf("/")+1);
		addFilter(url, docShell.forcedCharset.toString());
		saveFilterList();
	}
}

function observeFilterList() {
	const prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2).QueryInterface(Ci.nsIPrefService);
	prefService.addObserver("extensions.charsetswitcher.filterlist", { observe: loadFilterList }, false);
}

loadFilterList();
observeFilterList();
