const Ci = Components.interfaces;
var CS = {
	loadCharsetList: function() {
		Components.classes["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService)
		.notifyObservers(null, "charsetmenu-selected", "other");
	},
	getFocusedWinDocShell: function() {
		var winmgr = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService().QueryInterface(Ci.nsIWindowMediator);
		var topwin = winmgr.getMostRecentWindow("navigator:browser");
		var wnd = topwin.document.commandDispatcher.focusedWindow;
		if ((topwin == wnd) || (wnd == null)) wnd = topwin.content;
		return wnd.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIWebNavigation).QueryInterface(Ci.nsIDocShell);
	},
	getFocusedWinCharset: function() {
		return this.getFocusedWinDocShell().QueryInterface(Ci.nsIDocCharset).charset;
	},
	getFocusedWinAddress: function() {
		var uri = this.getFocusedWinDocShell().currentURI;
		if (!uri || (uri.scheme != "http" && uri.scheme != "https")) return "";
		return uri.spec.substring(0, uri.spec.lastIndexOf("/")+1);
	},
	addRule: function(urlrule, charset) {
		var rules = document.getElementById("filter-childs");
		var item = document.createElement("treeitem");
		var row = document.createElement("treerow");
		var c1 = document.createElement("treecell");
		var c2 = document.createElement("treecell");
		c1.setAttribute("label", urlrule);
		c2.setAttribute("label", charset);
		row.appendChild(c1);
		row.appendChild(c2);
		item.appendChild(row);
		rules.appendChild(item);
		return (rules.childNodes.length-1);
	},
	loadPrefs: function() {
		document.getElementById("charset-box").menuBoxObject.openMenu(true);
		document.getElementById("charset-box").menuBoxObject.openMenu(false);
		//
		var list = nsPreferences.copyUnicharPref("extensions.charsetswitcher.filterlist", "").split(" ");
		var rules = document.getElementById("filter-childs");
		while (rules.hasChildNodes()) rules.removeChild(rules.firstChild);
		for (var i=0; i<list.length; i++) {
			if (list[i] != "") {
				var item = list[i].split(",");
				this.addRule(item[0], item[1]);
			}
		}
		this.onRuleSelected();
	},
	savePrefs: function() {
		var list = [];
		var filter = document.getElementById("filter-list");
		var count = filter.view.rowCount;
		for (var i=0; i<count; i++) {
			var urlrule = filter.view.getCellText(i, filter.columns["urlrule-col"]);
			var charset = filter.view.getCellText(i, filter.columns["charset-col"]);
			var item = urlrule + "," + charset;
			list.push(item);
		}
		list.sort();
		nsPreferences.setUnicharPref("extensions.charsetswitcher.filterlist", list.join(" "));
	},
	onRuleSelected: function() {
		var filter = document.getElementById("filter-list");
		var disabled = (filter.view.selection.count >= 2);
		document.getElementById("urlrule-box").disabled = disabled;
		document.getElementById("charset-box").disabled = disabled;
		document.getElementById("charset-box").selectedIndex = -1;
		var urlrule = "";
		var charset = "";
		switch (filter.view.selection.count) {
		case 0:
			urlrule = this.getFocusedWinAddress();
			charset = this.getFocusedWinCharset();
			break;
		case 1:
			urlrule = filter.view.getCellText(filter.currentIndex, filter.columns["urlrule-col"]);
			charset = filter.view.getCellText(filter.currentIndex, filter.columns["charset-col"]);
			break;
		}
		document.getElementById("urlrule-box").value = urlrule;
		document.getElementById("charset-box").value = charset;
		document.getElementById("delete-btn").disabled = (filter.view.selection.count == 0);
		document.getElementById("addnew-btn").disabled = (filter.view.selection.count >= 2);
	},
	findRule: function(value) {
		var filter = document.getElementById("filter-list");
		for (var i=filter.view.rowCount-1; i>=0; i--) {
			var rule = filter.view.getCellText(i, filter.columns["urlrule-col"]);
			if (rule == value) break;
		}
		return i;
	},
	onAddnewCommand: function() {
		var filter = document.getElementById("filter-list");
		var urlrule = document.getElementById("urlrule-box").value;
		var charset = document.getElementById("charset-box").value;
		urlrule = urlrule.replace(/^\s+/g,"").replace(/\s+$/g,"");
		if (!urlrule) return;
		var idx = this.findRule(urlrule);
		if (idx == -1) idx = this.addRule(urlrule, charset);
		filter.view.selection.select(idx);
		filter.boxObject.ensureRowIsVisible(idx);
		filter.focus();
		document.getElementById("charset-box").value = charset;
	},
	onDeleteCommand: function() {
		var filter = document.getElementById("filter-list");
		var rules = document.getElementById("filter-childs");
		filter.view.selection.selectEventsSuppressed = true;
		for (var i=rules.childNodes.length-1; i>=0; i--) {
			if (filter.view.selection.isSelected(i))
				rules.removeChild(rules.childNodes[i]);
		}
		filter.view.selection.selectEventsSuppressed = false;
		this.onRuleSelected();
	},
	onModifyCommand: function() {
		var filter = document.getElementById("filter-list");
		if (filter.view.selection.count == 1) {
			filter.view.setCellText(filter.currentIndex, filter.columns["urlrule-col"], document.getElementById("urlrule-box").value);
			filter.view.setCellText(filter.currentIndex, filter.columns["charset-col"], document.getElementById("charset-box").value);
		}
	}
}
