const Ci = Components.interfaces;
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function CharsetSwitcher() {
	Components.utils.import("resource://charsetswitcher/charsetswitcher.jsm", this);
}
CharsetSwitcher.prototype = {
	classDescription: "CharsetSwitcher Component",
	classID: Components.ID("{C67764AE-74F1-40c2-9D90-336FC0D3375C}"),
	contractID: "@moztw.org/charsetswitcher;1",
	QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentPolicy, Ci.nsIChannelEventSink]),

	// common function
	_getDocShell: function(aWindow) {
		return aWindow.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIWebNavigation).QueryInterface(Ci.nsIDocShell);
	},

	// nsIContentPolicy implementation
	shouldLoad: function(contentType, newLocation, requestOrigin, aContext, mimeTypeGuess, extra) {
		try {
			if ((contentType != Ci.nsIContentPolicy.TYPE_DOCUMENT) && (contentType != Ci.nsIContentPolicy.TYPE_SUBDOCUMENT)) {
				return Ci.nsIContentPolicy.ACCEPT;
			}
			if (!newLocation || (newLocation.scheme != "http" && newLocation.scheme != "https")) {
				return Ci.nsIContentPolicy.ACCEPT;
			}
			if (!aContext || !aContext.contentWindow) {
				return Ci.nsIContentPolicy.ACCEPT;
			}
			var docShell = this._getDocShell(aContext.contentWindow);
			if (!docShell) {
				return Ci.nsIContentPolicy.ACCEPT;
			}
			//XXX: ready to determine charset
			this.determineCharset(docShell, newLocation);

		} catch(e) {
			Components.utils.reportError(e);
		}
		return Ci.nsIContentPolicy.ACCEPT;
	},
	shouldProcess: function(contentType, contentLocation, requestOrigin, insecNode, mimeType, extra) {
		return Ci.nsIContentPolicy.ACCEPT;
	},
	// nsIContentPolicy end

	// nsIChannelEventSink implementation
	onChannelRedirect: function(oldChannel, newChannel, flags) {
		try {
			if (!(newChannel.loadFlags & Ci.nsIChannel.LOAD_DOCUMENT_URI)) {
				return;
			}
			var newLocation = newChannel.URI;
			if (!newLocation || (newLocation.scheme != "http" && newLocation.scheme != "https")) {
				return;
			}
			var docShell;
			var callbacks = [];
			if (newChannel.notificationCallbacks) {
				callbacks.push(newChannel.notificationCallbacks);
			}
			if (newChannel.loadGroup && newChannel.loadGroup.notificationCallbacks) {
				callbacks.push(newChannel.loadGroup.notificationCallbacks);
			}
			for each (var callback in callbacks) {
				try {
					docShell = this._getDocShell(callback.getInterface(Ci.nsILoadContext).associatedWindow);
					break;
				} catch(e) {}
			}
			if (!docShell) {
				return;
			}
			//XXX: ready to determine charset
			this.determineCharset(docShell, newLocation);

		} catch(e) {
			Components.utils.reportError(e);
		}
	},
	asyncOnChannelRedirect: function(oldChannel, newChannel, flags, redirectCallback) {
		this.onChannelRedirect(oldChannel, newChannel, flags);
		redirectCallback.onRedirectVerifyCallback(0);
	}
	// nsIChannelEventSink end
}
const NSGetFactory = XPCOMUtils.generateNSGetFactory([CharsetSwitcher]);
